//*****************************************************************************
//*
//*
//*		NetStatLibRemote.cpp
//*
//*
//*****************************************************************************
//
//	Source-Code Beispiel:
//
//		NetStatEntry	*pList;
//		unsigned		 uPos;
//
//			pList = NetStatGet();
//		if(!pList)return;
//
//		for(uPos=0;;uPos++)
//			{
//			if(pList[uPos].ucType==0xFF)break;
//
//			printf("%s:%i -> %s:%i = %s\n",
//						pList[uPos].cLocalAddr ,pList[uPos].usLocalPort ,
//						pList[uPos].cRemoteAddr,pList[uPos].usRemotePort,
//						NetStatState(pList[uPos].ucState));
//			}
//
//		NetStatFree(pList);
//
#include	<windows.h>
#include	<string.h>
#include	<stdlib.h>
#include	<stdio.h>

#include	"AZ_Lib.h"
#include	"NetStatLib.h"
#include	"../AzCommLib/Ssh/SshClient.h"
#include	"../AzCommLib/Raw/RawClient.h"
#include	"../AzCommLib/Telnet/TelnetClient.h"

#define 	v	volatile

#define 	SYNCH_TYPE		CRITICAL_SECTION
#define 	SYNCH_CREATE()	InitializeCriticalSection(&sNetStatSynch);
#define 	SYNCH_DELETE()	DeleteCriticalSection(&sNetStatSynch);
#define 	UNLOCK()		LeaveCriticalSection(&sNetStatSynch)
#define 	LOCK()			EnterCriticalSection(&sNetStatSynch)
#define 	MAX_ITEMS		4096

static	char				cConnServer[256];
static	char				cConnLogin [256];
static	char				cConnPasswd[256];
static	void			   *hCommHandle =  0;
static	int					iConnTout	=  5000;
static	int					iConnMode	= -1;
static	int					iConnPort	=  0;
static	int	volatile		iThreadRun	=  0;
static	int	volatile		iThreadStop	=  0;
static	int	volatile		iRequActive	=  0;
static	int	volatile		iRequTicker	=  0;
static	int	volatile		iRequFilter =  0;
static	int	volatile		iItemCount	=  0;
static	int					iInitDone	=  0;
static	SYNCH_TYPE			sNetStatSynch;
static	HANDLE				hEventRequ	=  0;
static	HANDLE				hEventResp	=  0;
static	NetStatEntry		aItemList[MAX_ITEMS];
static	CStringChain		sPidList(sizeof(int)*2);
static	CStringChain		sTmpList(sizeof(int)*2);
static	int					aNull[3];




static	int   (__cdecl  *ProcErrorText  )(void *hHandle,int iError,char *pBuffer,int iSize);
static	int   (__cdecl  *ProcConnect    )(void *hHandle,const char *pAddress,int iPort,const char *pUser,const char *pPassword,int iTimeout);
static	int   (__cdecl  *ProcWait       )(void *hHandle,int  timeout,int mode);
static	int   (__cdecl  *ProcRead       )(void *hHandle,void       *pBuffer,int iSize);
static	int   (__cdecl  *ProcWrite      )(void *hHandle,const void *pData  ,int iSize);
static	int   (__cdecl  *ProcReadCount  )(void *hHandle,int *pLine);
static	int   (__cdecl  *ProcWriteCount )(void *hHandle);
static	int   (__cdecl  *ProcIsConnected)(void *hHandle);
static	int   (__cdecl  *ProcClose      )(void *hHandle);
static	void  (__cdecl  *ProcBreak      )(void *hHandle);
static	void *(__cdecl  *ProcCreate     )();
static	DWORD WINAPI	 NetStatThread  (LPVOID pParam);
static	int 			 NetStatRemoveEsc (char *pLine);
static	void			 NetStatDecodeLine(char *pLine);


//*****************************************************************************
//*
//*		NetStatState
//*
//*****************************************************************************
const char *NetStatState(unsigned uState)
{

	switch(uState)
		{
	case NSL_TCP_STATE_UDP:			return "UDP";
	case NSL_TCP_STATE_CLOSED:		return "CLOSED";
	case NSL_TCP_STATE_LISTEN:		return "LISTEN";
	case NSL_TCP_STATE_SYN_SENT:	return "SYN SENT";
	case NSL_TCP_STATE_SYN_RCVD:	return "SYN RECIVED";
	case NSL_TCP_STATE_ESTAB:		return "ESTABLISHED";
	case NSL_TCP_STATE_FIN_WAIT1:	return "FIN WAIT1";
	case NSL_TCP_STATE_FIN_WAIT2:	return "FIN WAIT2";
	case NSL_TCP_STATE_CLOSE_WAIT:	return "CLOSE WAIT";
	case NSL_TCP_STATE_CLOSING:		return "CLOSING";
	case NSL_TCP_STATE_LAST_ACK:	return "LAST ACK";
	case NSL_TCP_STATE_TIME_WAIT:	return "TIME WAIT";
	case NSL_TCP_STATE_DELETE_TCB:	return "DELETE";
		}

return "\?\?\?";
}

//*****************************************************************************
//*
//*		NetStatFree
//*
//*****************************************************************************
// 	Die Verbindungsinformationen wieder freigeben
void NetStatFree(NetStatEntry *pList)
{
unsigned	uPos;


	if(!pList)return;

	for(uPos=0;pList[uPos].ucType!=0xFF;uPos++)
		{
		if(pList[uPos].usReserved==0)continue;
		delete pList[uPos].pProcessPath;
		}

	delete pList;

}

//*****************************************************************************
//*
//*		NetStatGet
//*
//*****************************************************************************
// 	Holt eine Liste mit Verbindungsinformationen holen
// 	uFilter		: Filterbits (0=TCP 1=UDP 2=TCPv6 3=UDPv6 16=Pfade auflsen)
// 	pCount		: Hier wird die Anzahl der gefundenen Eintrge gespeichert
// 	Ergibt einen Zeiger auf die Liste mit den Eintrgen
NetStatEntry *NetStatGet(unsigned uFilter,unsigned *pCount)
{
static int		iLocked=0;
NetStatEntry   *pList;
NetStatEntry   *pItem;
int			   *pData;
int				iNum;
int				iIdx;
int				iPos;



	if(iLocked)
		{
		if(pCount)*pCount=0;
		return 0;
		}

	if(iConnMode<0)												// Die letzten Daten wenn nicht verbunden
		{
		goto Copy;
		}


	LOCK();

	iRequFilter = uFilter;										// Anfrage versenden
	iRequActive = 1;
	iRequTicker = 0;
	SetEvent(hEventRequ);

	UNLOCK();


	if(WaitForSingleObject(hEventResp,80000)!=WAIT_OBJECT_0)	// Warte auf eine neue Anfrage
		{
		LOCK();
		iRequActive = 0;
		ResetEvent(hEventRequ);									// Antwort Event rcksetzen
		UNLOCK();

		if(pCount)*pCount=0;

		return 0;
		}


Copy:

	LOCK();

	pList = new NetStatEntry[iItemCount+1];

	if((uFilter&0x0F)==0x0F)									// Die ganze Liste kopieren
		{
		memcpy(pList,aItemList,sizeof(NetStatEntry)*iItemCount);
		iNum = iItemCount;
		}
	else{														// Die Liste filtern
		pItem = aItemList;

		for(iPos=0,iNum=0;iPos<iItemCount;iPos++,pItem++)
			{
			if(uFilter&(1<<pItem->ucType))
				{
				memcpy(pList+iNum,pItem,sizeof(NetStatEntry));
				iNum++;
				}
			}
		}

	pItem = pList;

	for(iPos=0;iPos<iNum;iPos++,pItem++)						// Absolute Pfade holen
		{
		pData	= ((int*)pItem->pProcessName)-2;
		iIdx	= pData[1];

		if(!iIdx)continue;

		pItem->pProcessPath	 = sTmpList[iIdx];
		pItem->pProcessName	 = pItem->pProcessPath;
		pItem->pProcessName	+= sTmpList.DataGet(iIdx);
		}

	memset(pList+iNum,0,sizeof(NetStatEntry));
	pList[iNum].ucType = 0xFF;

	ResetEvent(hEventResp);										// Antwort Event rcksetzen
	iRequActive = 0;
	iLocked		= 0;

	UNLOCK();



return pList;
}

//*****************************************************************************
//*
//*		NetStatPath
//*
//*****************************************************************************
// 	ndert den Pfad eine es Listeneintrages.
// 	pList		: Ist der Zeiger auf den Listeneintrag
// 	pPath		: Ist der neue Pfad
// 	uProcessId	: Ist die ID des Prozesses
void NetStatPath(NetStatEntry *pList,const char *pPath,unsigned uProcessId)
{
char	   *pNew;
char	   *pOld;
int			iLen;
int			iOff;


	if(!pList || !pPath)return;

	iLen = strlen(pPath);

	for(iOff=iLen;iLen>0;iOff--)								// Suche Dateinamen im Pfad
		{
		if(pPath[iOff-1]=='\\')break;
		if(pPath[iOff-1]=='/')break;
		}

	pOld	= pList->pProcessPath;
	pNew	= new char[iLen+1];
	memcpy(pNew,pPath,iLen+1);

	pList->pProcessPath		= pNew;
	pList->pProcessName		= pNew+iOff;

	if(pOld && pList->usReserved)
		{
		delete	pOld;
		}

	pList->usReserved = 1;
}



//*****************************************************************************
//*
//*		NetStatConnState
//*
//*****************************************************************************
//	Ergibt den aktuellen Zustand der Remote-Verbindung
//			0 = Keine Verbindung konfiguriert
//			1 = Verbindung ok
//			2 = Verbindung unterbrochen
int NetStatConnState()
{
int		iRet;


	if(iConnMode <0)return 0;
	if(!hCommHandle)return 0;

	LOCK();

	if(iConnMode>=0 && iConnMode<=2 && ProcIsConnected)
			iRet = ProcIsConnected(hCommHandle);
	else	iRet = 0;

	UNLOCK();


return (iRet)? 1:2;
};

//*****************************************************************************
//*
//*		NetStatDisconnect
//*
//*****************************************************************************
//	Bricht die aktuelle Verbindung ab
//	Ergibt 0 wenn ok oder eine beiFehlernummmer
int NetStatDisconnect()
{

	LOCK();

	if(iConnMode>=0 && ProcBreak && hCommHandle)
		{
		ProcBreak(hCommHandle);
		}

	iConnMode = -1;

	UNLOCK();

return 0;
}

//*****************************************************************************
//*
//*		NetStatConnect
//*
//*****************************************************************************
//	Setzt die Verbindungparameter
//	pServer		: Name des Servers
//	iPort		: Ist das TCP-Port (0=default)
//	pLogin		: Ist der Login-Namen fr die Verbindung
//	pPassword	: Ist das Passwort fr den Login
//	iMode		: Verbindungs Modus (0=SSH 1=Telnet 2=RAW)
//	iTiemout	: Timeout in Millisekunden
//	Ergibt 0 wenn ok oder eine beiFehlernummmer
int NetStatConnect(const char *pServer,int iPort,const char *pLogin,const char *pPassword,int iMode,int iTiemout)
{
HANDLE	hThread;
DWORD	uThread;


	if(iMode&0x80)
		{
		sPidList.Del();
		sTmpList.Del();
		sTmpList.Add(1,"");
		iMode &= ~0x80;
		}

	if(!pServer || !*pServer)return 1;
	if(iMode<0  || iMode>2  )return 2;

	LOCK();

	strlcpy(cConnServer,pServer  ,sizeof(cConnServer));
	strlcpy(cConnLogin ,pLogin   ,sizeof(cConnLogin ));
	strlcpy(cConnPasswd,pPassword,sizeof(cConnPasswd));

	iConnPort = iPort;
	iConnMode = iMode;
	iConnTout =(iTiemout>500)? iTiemout:5000;

	UNLOCK();

	if(!iThreadRun)
		{
		iThreadRun = 1;
		hThread    = CreateThread(NULL,0x40000,NetStatThread,NULL,0,&uThread);
		CloseHandle(hThread);
		}


return 0;
}

//*****************************************************************************
//*
//*		NetStatInit
//*
//*****************************************************************************
//	Wird aufgerufen um die API zu initialisieren
int NetStatInit()
{

	if(!iInitDone)
		{
		iInitDone = 1;
		SYNCH_CREATE();
		sTmpList.Add(1,"");
		hEventRequ = CreateEvent(NULL,TRUE,FALSE,NULL);
		hEventResp = CreateEvent(NULL,TRUE,FALSE,NULL);
		}

return 1;
}

//*****************************************************************************
//*
//*		NetStatExit
//*
//*****************************************************************************
//	Wird aufgerufen um die API zu deinitialisieren
int NetStatExit()
{

	while(iThreadRun)
		{
		iThreadStop = 1;
		Sleep(50);
		}


return 1;
}

//*****************************************************************************
//*
//*		NetStatAddPath
//*
//*****************************************************************************
//	Dekodiert eine empfangene Zeile
static void NetStatAddPath(int iLen,char *pName,int *pData)
{
char   *pText;
int		iPos;
int		iNum;


	LOCK();

	iPos  = sTmpList.Add(iLen,pName,CSC_END);
	pText = sTmpList.Get(iPos);
	iNum  = iLen-strdwlen(pText+iLen,0x00300000,iLen);

	sTmpList.DataSet(iPos,iNum  );
	sTmpList.DataSet(iPos,iPos,1);
	pData[1] = iPos;

	UNLOCK();

}

//*****************************************************************************
//*
//*		NetStatRemoveEsc
//*
//*****************************************************************************
//	Entferne ESC-Sequenzen aus einer Zeile
static int NetStatRemoveEsc(char *pLine)
{
char	cSign;
char	cMode;
int		iPos;
int		iOut;




	for(iPos=0;;iPos++)									// Suche erstes Zeichen
		{
		cMode = pLine[iPos];
		if(cMode==0   )return iPos;
		if(cMode==0x1B)break;
		if(cMode==0x7E)break;
		}

	iOut = iPos;


	for(iPos++;;iPos++)
		{
		cSign = pLine[iPos];


		if(cMode==0x7E)									// 7Eh
			{
			if(cSign!=0)iPos++;
			}
		else{											// ESC
			switch(pLine[iPos])
				{
			case 0:	pLine[iOut] = cSign;
					return iOut;

			case '[':

				for(iPos++;;iPos++)
					{				
					cSign = pLine[iPos];
					if(cSign>='0' && cSign<='9')continue;
					if(cSign==';'              )continue;
					break;
					}
				
				if(cSign!=0)iPos++;
				break;

			case ']':

				for(iPos++;;iPos++)
					{				
					cSign = pLine[iPos];
					if(cSign>='0' && cSign<='9')continue;
					break;
					}
				
				if(cSign!=0)iPos++;
				break;

			default:	

				iPos++;
				break;
				}	
			}			
						
		for(;;iPos++)
			{
			cMode = pLine[iPos];
			if(cMode==0x1B)break;
			if(cMode==0x7E)break;
			

			pLine[iOut] = cMode;
			iOut++;

			if(cMode==0)return iOut;
			}
		}

}

//*****************************************************************************
//*
//*		NetStatDecodeLine
//*
//*****************************************************************************
//	Dekodiert eine empfangene Zeile
static void NetStatDecodeLine(char *pLine)
{
NetStatEntry   *pItem;
int				aData[2];
int				iVal;
int 			iOff;
int				iPos;
int				iLen;
int				iNum;
int				iSub;


	if(iItemCount<0 || iItemCount>=MAX_ITEMS)
		{
		return;
		}

	pItem = aItemList+iItemCount;

	if(!memicmp(pLine,"tcp",3))							// Art der Verbindung
		{
		pItem->ucType = 0;
		}
	else if(!memicmp(pLine,"udp",3))
		{
		pItem->ucType = 1;
		}
	else{
		return;
		}

	pItem->usReserved	= 0;

	pLine += strwlen(pLine,0x1F);

	for(iPos=0;iPos<2;iPos++)							// Erste zwei Spalten berspringen
		{
		pLine += strwlen(pLine,0xA0);
		iNum   = strwlen(pLine,0x01);

		if(iNum && strwchar(pLine[iNum],0x00400207))
			{
			break;
			}

		pLine += iNum;
		}


	pLine += strwlen (pLine,0xA0);
	iNum   = strcwlen(pLine,0xA0);						// Lokale Adresse scannen
	iSub   = strbwlen(pLine+iNum,0x01,iNum);
	iLen   = iNum-iSub-1;

	if(iNum<3 || iLen<0 || iSub==0 || pLine[iLen]!=':')
		{
		return;
		}

	   iVal = atoi(pLine+iLen+1);
	if(iVal<=0 || iVal>0xFFFF)
		{
		return;
		}

	pItem->usLocalPort = iVal;
	pLine[iLen] = 0;
	iPos = strwlen(pLine,0x07);

	if(pLine[iPos]!='.')								// IPV6 Typ
		{
		pItem->ucType+=2;
		}

	strlcpy(pItem->cLocalAddr,pLine,sizeof(pItem->cLocalAddr));

	pLine += iNum;
	pLine += strwlen (pLine,0xA0);
	iNum   = strcwlen(pLine,0xA0);						// Remote-Adresse scannen
	iSub   = strbwlen(pLine+iNum,0x00800001,iNum);
	iLen   = iNum-iSub-1;

	if(iNum<3 || iLen<0 || iSub==0 || pLine[iLen]!=':')
		{
		return;
		}

	   iVal = atoi(pLine+iLen+1);
	if(iVal>0xFFFF)
		{
		return;
		}

	pItem->usRemotePort = iVal;
	pLine[iLen] = 0;

	strlcpy(pItem->cRemoteAddr,pLine,sizeof(pItem->cRemoteAddr));

	pLine += iNum;
	pLine += strwlen(pLine,0xA0);
	iNum   = strwlen(pLine,0x07);


	if(pItem->ucType&0x01)								// Status scannen
		{
		pItem->ucState = NSL_TCP_STATE_UDP;
		pItem->cRemoteAddr[0] = 0;
		}
	else if(iNum==6 && !memicmp(pLine,"CLOSED",6))
		{
		pItem->ucState = NSL_TCP_STATE_CLOSED;
		}
	else if(iNum==6 && !memicmp(pLine,"LISTEN",6))
		{
		pItem->ucState = NSL_TCP_STATE_LISTEN;
		}
	else if(iNum==8 && !memicmp(pLine,"SYN_SENT",8))
		{
		pItem->ucState = NSL_TCP_STATE_SYN_SENT;
		}
	else if(iNum==8 && !memicmp(pLine,"SYN_RECV",8))
		{
		pItem->ucState = NSL_TCP_STATE_SYN_RCVD;
		}
	else if(iNum==8 && !memicmp(pLine,"LAST_ACK",8))
		{
		pItem->ucState = NSL_TCP_STATE_LAST_ACK;
		}
	else if(iNum==9 && !memicmp(pLine,"FIN_WAIT1",9))
		{
		pItem->ucState = NSL_TCP_STATE_FIN_WAIT1;
		}
	else if(iNum==9 && !memicmp(pLine,"FIN_WAIT2",9))
		{
		pItem->ucState = NSL_TCP_STATE_FIN_WAIT2;
		}
	else if(iNum==10 && !memicmp(pLine,"CLOSE_WAIT",10))
		{
		pItem->ucState = NSL_TCP_STATE_CLOSE_WAIT;
		}
	else if(iNum==11 && !memicmp(pLine,"ESTABLISHED",11))
		{
		pItem->ucState = NSL_TCP_STATE_ESTAB;
		}
	else if(iNum==11 && !memicmp(pLine,"ESTABLISHED",11))
		{
		pItem->ucState = NSL_TCP_STATE_ESTAB;
		}
	else if(iNum==7 && !memicmp(pLine,"CLOSING",7))
		{
		pItem->ucState = NSL_TCP_STATE_CLOSE_WAIT;
		}
	else{
		pItem->ucState = NSL_TCP_STATE_UNKNOWN;
		}

	if(!strwchar(pLine[0],0x01))						// Kein Statustext
		{
		pLine += iNum;
		pLine += strwlen(pLine,0xA0);
		iNum   = strwlen(pLine,0x01);
		}

	if(strwchar(pLine[iNum],0xA0))						// Zahlenspalten berspringen
		{
		pLine += iNum;
		pLine += strwlen(pLine,0xA0);
		iNum   = strwlen(pLine,0x01);

		if(strwchar(pLine[iNum],0xA0))
			{
			pLine += iNum;
			pLine += strwlen(pLine,0xA0);
			iNum   = strwlen(pLine,0x01);
			}
		}


	pItem->uProcessId = atoi(pLine); 					// PID scannen

	pLine += iNum;

	if(*pLine=='/')
		{
		pLine++;

		if(pLine[0]=='-' && pLine[1]=='>')				// <PID>:-><..Path..>
			{
			pLine += 2;
			iVal   = 1;
			}
		else{
			iVal   = 0;
			}

		if(pLine[0]=='"')								// "..Path.."
			{
			pLine++;
			iNum  = strchar (pLine,'"');
			iNum -= strbwlen(pLine+iNum,0x40,iNum);
			}
		else{											// /..Path..
			iNum  = strwlen(pLine,(iVal)? 0x0238021F:0x0208021F);
			}

		iOff  = iNum;
		iOff -= strdwlen(pLine+iNum,0x00300000,iNum);

		   iPos = sPidList.FindFast(pItem->uProcessId,iNum-iOff,pLine+iOff);
		if(iPos<0)
			{
			aData[0] = pItem->uProcessId;
			aData[1] = 0;

			if(iVal)NetStatAddPath(iNum,pLine,aData);

			iPos	 = sPidList.Add(iNum-iOff,pLine+iOff,CSC_UABC,aData);
			}

		pItem->pProcessName = sPidList[iPos];
		pItem->pProcessPath = sPidList[iPos];
		}
	else{
		pItem->pProcessName = (char*)(aNull+2);
		pItem->pProcessPath = (char*)(aNull+2);
		}


	iItemCount++;

}

//*****************************************************************************
//*
//*		NetStatThread
//*
//*****************************************************************************
//	Thread fr die Remote-Verbindung
static DWORD WINAPI	NetStatThread(LPVOID pParam)
{
int				iLastMode=-1;
char			aData[4096];
char			aStop[20];
void		   *hHandle;
NetStatEntry   *pItem;
const char	   *pText;
int			   *pData;
int				iLine;
int				iLen;
int				iRet;
int				iPos;
int				iNum;
int				iOk;


	hHandle		= 0;
	iOk			= 0;

	while(!iThreadStop)
		{
		if(iRequActive)									// Anfrage abbrechen
			{
			LOCK();

			if(iRequActive)
				{
				   iRequTicker++;
				if(iRequTicker==3)
					{
					iItemCount = 0;
					ResetEvent(hEventRequ);
					SetEvent  (hEventResp);
					}
				}

			UNLOCK();
			}


		if(iLastMode!=iConnMode)						// Hat sich der Modus verndert
			{
			LOCK();

			if(iLastMode>=0 && hHandle)
				{
				ProcClose(hHandle);
				}

			iLastMode   = iConnMode;
			hCommHandle = NULL;
			hHandle     = NULL;
			iOk			= 0;

			if(iLastMode==0)							// SSH
				{
				ProcErrorText   = SshErrorText;
				ProcConnect     = SshConnect;
				ProcWait        = SshWait;
				ProcRead        = SshRead;
				ProcWrite       = SshWrite;
				ProcIsConnected = SshIsConnected;
				ProcWriteCount  = SshWriteCount;
				ProcReadCount   = SshReadCount;
				ProcClose       = SshClose;
				ProcBreak       = SshBreak;
				ProcCreate      = SshCreate;
				}

			if(iLastMode==1)							// Telnet
				{
				ProcErrorText   = TelnetErrorText;
				ProcConnect     = TelnetConnect;
				ProcWait        = TelnetWait;
				ProcRead        = TelnetRead;
				ProcWrite       = TelnetWrite;
				ProcIsConnected = TelnetIsConnected;
				ProcWriteCount  = TelnetWriteCount;
				ProcReadCount   = TelnetReadCount;
				ProcClose       = TelnetClose;
				ProcBreak       = TelnetBreak;
				ProcCreate      = TelnetCreate;
				}

			if(iLastMode==2)							// RAw
				{
				ProcErrorText   = RawErrorText;
				ProcConnect     = RawConnect;
				ProcWait        = RawWait;
				ProcRead        = RawRead;
				ProcWrite       = RawWrite;
				ProcIsConnected = RawIsConnected;
				ProcWriteCount  = RawWriteCount;
				ProcReadCount   = RawReadCount;
				ProcClose       = RawClose;
				ProcBreak       = RawBreak;
				ProcCreate      = RawCreate;
				}

			UNLOCK();
			}


		if(iLastMode<0 || iLastMode>2)					// Kein Modus gewhlt
			{
			Sleep(250);
			continue;
			}


		if(!iOk)										// Ist die Verbindung noch ok
			{
			if(hHandle)
				{
				LOCK();
				hCommHandle = NULL;
				UNLOCK();

				ProcClose(hHandle);
				iOk = 0;
				}

				hHandle = ProcCreate();
			if(!hHandle)
				{
				Sleep(250);
				iOk = 0;
				continue;
				}

			LOCK();
			hCommHandle = hHandle;
			UNLOCK();

			   iRet = ProcConnect(hHandle,cConnServer,iConnPort,(cConnLogin[0])? cConnLogin:NULL,(cConnPasswd[0])? cConnPasswd:NULL,iConnTout+5000);
			if(iRet)
				{
				Sleep(250);
				continue;
				}

			ProcWrite(hHandle,"force_color_prompt=no\n",22);
			
			iOk = 1;
			}

		if(!ProcIsConnected(hHandle))					// Die Verbindung prfen
			{
			iOk = 0;
			Sleep(250);
			continue;
			}
														// Warte auf eine neue Anfrage
		if(WaitForSingleObject(hEventRequ,500)!=WAIT_OBJECT_0)
			{
			continue;
			}


		ResetEvent(hEventRequ);

		iItemCount	= 0;
		pText		= "\nnetstat -a -n -p -t -u\necho \"?a@b?b@c\"\n";
		iLen  		= strlen(pText);
		iRet  		= ProcWrite(hHandle,pText,iLen);

		if(iLen!=iRet)
			{
			iOk = 0;
			continue;
			}

		for(;;)											// Zeilenweise empfangen
			{
				iRet = ProcWait(hHandle,iConnTout,1);
			if(!iRet)break;
			    iRet = ProcReadCount(hHandle,&iLine);
			if( iRet<=0)break;

			if(iLine)iRet=iLine;

			if(iRet>=sizeof(aData))
				{
				iRet = sizeof(aData)-1;
				}

			   iLen = ProcRead(hHandle,aData,iRet);		// Die nchste Zeile einlesen
			if(iLen<0)
				{
				iOk = 0;
				break;
				}

			aData[iLen] = 0;

			iNum = strcwlen(aData,0x01000000);
														// Letzte Zeile empfangen
			if(aData[iNum]=='?' && !memcmp(aData+iNum,"?a@b?b@c",8) && strfind(aData,"echo")<0)
				{
				break;
				}

			NetStatRemoveEsc (aData);
			NetStatDecodeLine(aData);
			}

		if(iOk)											// Empfangspuffer leeren
			{
			for(;;)
				{
				   iRet = ProcReadCount(hHandle,0);
				if(iRet<=0)break;
				if(iRet>sizeof(aData))iRet=sizeof(aData);
				ProcRead(hHandle,aData,iRet);
				}
			}


		if(!(iRequFilter&0x10000))						// Keine Pfade auflsen
			{
			LOCK();

			if(iRequActive)								// Anfrage besttigen
				{
				SetEvent(hEventResp);
				}

			UNLOCK();

			continue;
			}

		pItem = aItemList;

		for(iPos=0;iPos<iItemCount;iPos++,pItem++)		// Suche die Pfade
			{
			   pData = ((int*)pItem->pProcessName)-2;
			if(pItem->usReserved     !=0)continue;
			if(pItem->pProcessName[0]==0)continue;
			if(pData[1]              !=0)continue;

			iLen = sprintf(aStop,"?x?%010i?y?",pItem->uProcessId);
			iLen = sprintf(aData,"ls -al /proc/%i/exe \necho \"%s\"\n",pItem->uProcessId,aStop);
			iRet = ProcWrite(hHandle,aData,iLen);

			if(iLen!=iRet)
				{
				iOk = 0;
				break;
				}

			for(;;)										// Zeilenweise empfangen
				{
					iRet = ProcWait(hHandle,iConnTout,1);
				if(!iRet)break;
					iRet = ProcReadCount(hHandle,&iLine);
				if( iRet<=0)break;

				if(iLine)iRet=iLine;

				if(iRet>=sizeof(aData))
					{
					iRet = sizeof(aData)-1;
					}

				   iLen = ProcRead(hHandle,aData,iRet);	// Die nchste Zeile einlesen
				if(iLen<0)
					{
					iOk = 0;
					break;
					}

				aData[iLen] = 0;

				iNum = strcwlen(aData,0x01000000);
														// Letzte Zeile empfangen
				if(aData[iNum]=='?' && !memcmp(aData+iNum,aStop,16) && strfind(aData,"echo")<0)
					{
					break;
					}

				   iNum = strfind(aData,"->");
				if(iNum<=0)continue;					// Ist ein Link vorhanden

				iLen  = NetStatRemoveEsc(aData+iNum)+iNum;

				iNum += 2;
				iNum += strwlen (aData+iNum,0xA0);
				iLen -= strbwlen(aData+iLen,0xE0,iLen-iNum);
				iLen -= iNum;

				if(iLen<=0)continue;

				NetStatAddPath(iLen,aData+iNum,pData);
				}
			}

		LOCK();

		if(iRequActive)									// Anfrage besttigen
			{
			SetEvent(hEventResp);
			}

		UNLOCK();
		}



	LOCK();

	if(iRequActive)										// Anfrage besttigen
		{
		iItemCount = 0;
		SetEvent(hEventResp);
		}

	if(iLastMode>=0 && hHandle)							// Verbindung schlieen
		{
		ProcClose(hHandle);
		hCommHandle = NULL;
		}

	iThreadRun = 0;

	UNLOCK();


return 0;
}

